/* ----------------------------------------------------------------------------
    NSTX -- tunneling network-packets over DNS

     (C) 2000 by Florian Heinz and Julien Oster

    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software
    Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.

  -------------------------------------------------------------------------- */

#include <sys/socket.h>
#include <sys/types.h>
#include <string.h>
#include <netinet/in.h>
#include <unistd.h>
#include <errno.h>
#include <stdio.h>
#include <fcntl.h>
#include <stdlib.h>
#include "nstx.h"
#include "nstx_pstack.h"
#include "nstx_dns.h"

extern int nsfd;
extern struct nstx_senditem * nstx_get_senditem(void);

struct nshdr {
   unsigned short id;
//   unsigned char  qr:1;
//   unsigned char  opcode:4;
//   unsigned char  aa:1;
//   unsigned char  tc:1;
//   unsigned char  rd:1;
//   unsigned char  ra:1;
//   unsigned char  z:3;
//   unsigned char  rcode:4;
   unsigned char hdr1;
   unsigned char hdr2;
   unsigned short qdcount;
   unsigned short ancount;
   unsigned short nscount;
   unsigned short arcount;
};

struct txthdr {
   unsigned short type;
   unsigned short class;
   unsigned int ttl;
   unsigned short rdlength;
};

int nstx_server_send_packet (char *buf, int buflen, struct sockaddr_in *peer) {
   int hlen, count, totlen = 0, blafd, senditemlen;
   char *sendbuf, *nb, *name, *ptr, *qs;
   struct nshdr *ns, newns;
   struct txthdr txt;
   struct nstx_senditem * senditem;
   
   struct nstx_senditem *dummysenditem;
   struct nstxhdr dummyhdr;
   
   ns = (struct nshdr *) buf;
   if (nsfd < 0)
     return -1;
   
   sendbuf = malloc(3000);
   
   senditem = nstx_get_senditem();
   
   if (!senditem) {
      dummysenditem = malloc(sizeof(struct nstx_senditem));
      dummyhdr.magic = NSTX_MAGIC;
      dummyhdr.seq = dummyhdr.frc = 0;
      dummyhdr.id = 0x00;
      dummyhdr.crop = 0;
      dummyhdr.flags = 0x00;
      *dummysenditem->data = sizeof(struct nstxhdr);
      memcpy(dummysenditem->data+1, &dummyhdr, sizeof(struct nstxhdr));
      senditem = dummysenditem;
   }
   
   senditemlen = (int) *senditem->data;
   
   /***************************/
   /*       Getting name      */
   nb = buf + 12;
   hlen = 0;
   do {
      count = *nb;
      hlen += count + 1;
      nb += count + 1;
   } while (count);
   name = malloc(hlen);
   memset(name, 0, hlen);
   memcpy(name, buf+12, hlen);
   qs = malloc(hlen+4);
   memcpy(qs, buf+12, hlen+4);
   totlen += 1;
   totlen += hlen+4;
   /***************************/
   
   
   newns.id = ns->id;
//   newns.qr = 1;
//   newns.opcode = 0;
//   newns.aa = 1;
//   newns.tc = 0;
//   newns.rd = 0;
//   newns.ra = 0;
//   newns.z =  0;
//   newns.rcode = 0;
   newns.hdr1 = 0x84;
   newns.hdr2 = 0x00;
   newns.qdcount = htons(1);
   newns.ancount = htons(1);
   newns.nscount = 0x00;
   newns.arcount = 0x00;
   totlen += sizeof(struct nshdr);
   
   txt.type = htons(16);
   txt.class = htons(1);
   txt.ttl = htonl(0);
   txt.rdlength = htons(senditemlen + 1);
   totlen += sizeof(struct txthdr);

   ptr = sendbuf;
   memcpy(ptr, (char *) &newns, sizeof(struct nshdr));
   ptr += sizeof(struct nshdr);
   
   memcpy(ptr, qs, hlen+4);
   ptr += hlen+4;
   
//   memcpy(ptr, name, hlen);
//   ptr += hlen;

   *ptr = 192;
   ptr++;
   *ptr = sizeof(struct nshdr);
   ptr++;

   memcpy(ptr, (char *) &txt, sizeof(struct txthdr));
   ptr += sizeof(struct txthdr) - 2;
   
   memcpy(ptr, senditem->data, senditemlen + 1);
   totlen += senditemlen + 2;
   
   blafd = open("/tmp/bla", O_CREAT|O_WRONLY|O_TRUNC);
   write(blafd, sendbuf, totlen-2);
   close(blafd);
   
   if (sendto(nsfd,sendbuf,totlen-2,0,peer,sizeof(struct sockaddr_in)) < 0) {
      perror("nstx_server_send_packet.sendto");
      exit(1);
   }
   free(name);
   free(senditem);
   free(sendbuf);
   free(qs);
   return 0;
}
