/* ----------------------------------------------------------------------------
    NSTX -- tunneling network-packets over DNS

     (C) 2000 by Julien Oster and Florian Heinz

    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software
    Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.

  -------------------------------------------------------------------------- */

#include <string.h>
#include <stdlib.h>

#include "nstx.h"

int nstx_build_fqdn(char * suffix, char *newbuf, char *buf, int len) {
  int alen, blen;
  int i;
  char abuf[255], bbuf[255];
  char *p, *p2;

  memcpy (abuf, buf, alen = len);

  if (len % 3)
    alen = len + (3 - (len % 3));

  if (alen > HCHUNKLEN)
    return -1;

  for (i = len; i < alen; i++)
    abuf[i] = 0;

  blen = nstx_encode (bbuf, abuf, alen);
  bbuf[blen] = 0;

  alen = nstx_decode (abuf, bbuf, blen);

  p = bbuf;
  p2 = newbuf;
  while (*p) {
    if (p - bbuf && !((p - bbuf) % 63))
      *(p2++) = '.';
    if (! *p)
      break;
    *(p2++) = *(p++);
  };

  *p2 = 0;
  i = p2-newbuf;

  if ((i += strlen(suffix)) > FQDN_MAXLEN)
    return -3;

  strcat (newbuf, suffix);

  return i;
}

char *nstx_remove_dots (char *fqdn) {
  char *retfqdn;
  char *p, *p2;

  p = fqdn;
  p2 = retfqdn = malloc(strlen(fqdn)+1);

  while (*p) {
    if (*p != '.')
      *(p2++) = *(p++);
    else
      p++;
  }

  return retfqdn;
}

int nstx_string_to_name (char * newbuf, char * namestr, int len) {
  char * p = namestr;
  char * np = newbuf;
  char * pos;
  unsigned char npos;
  int i = 0;

  while ((pos = strchr (p, '.'))) {
    i += (npos = ((int)(pos-p)));

    if (len < i + 1)
      return -1;

    *(np++) = npos;
    memcpy (np, p, npos);
    p += npos + 1;
    np += npos;
  };

  npos = strlen (p);

  if (len < i+1)
    return -1;

  *(np++) = npos;
  memcpy (np, p, npos);
  np += npos;

  *np++ = 0;

  return np - newbuf;
}

int nstx_name_to_string (char * newbuf, char * splitted, int len) {
  char * p = splitted;
  int i = 0;

  if (len < i)
    return - 1;

  *newbuf = 0;

  while (*p) {
    i += *p;
    if (len < i + 1)
      return -1;

    strncat (newbuf, p+1, *p);
    p += (*p) + 1;

    if (*p) {
      i++;
      if (len < i + 1)
	return -1;
      strcat (newbuf, ".");
    }
  }

  p++;

  return p - splitted;
}
