/* ----------------------------------------------------------------------------
    NSTX -- tunneling network-packets over DNS

     (C) 2000 by Julien Oster and Florian Heinz

    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software
    Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.

  -------------------------------------------------------------------------- */

#include <stdio.h>
#include <stdlib.h>
#include <string.h>

#include <unistd.h>
#include <fcntl.h>

#include <sys/types.h>
#include <sys/socket.h>

#include <netinet/in.h>
#include <arpa/inet.h>

#include "nstx.h"
#include "nstx_dns.h"

static int sfd;

int nstxc_dns_init (struct sockaddr * sa, socklen_t salen,
		    struct sockaddr * myaddr, socklen_t myaddrlen) {

  if ((sfd = socket (AF_INET, SOCK_DGRAM, IPPROTO_UDP)) < 0)
    return -1;

  if (bind (sfd, myaddr, myaddrlen))
    return -3;

  if (connect (sfd, sa, salen))
    return -2;

  return sfd;
}

int nstxc_dns_query (char * name, unsigned short id) {
  struct dnsqueryhdr qh;
  char buf[255];
  char * p;
  int i;

  qh.id = ntohs(id);
  qh.qr = 0;
  qh.opcode = 0;
  qh.aa = 0;
  qh.tc = 0;
  qh.rd = 1;
  qh.ra = 0;
  qh.z = 0;
  qh.rcode = 0;

  qh.qdcount = htons(1);
  qh.ancount = 0;
  qh.nscount = 0;
  qh.arcount = 0;

  p = buf;

  memcpy (p, (char*)&qh, sizeof(qh));
  p += sizeof(qh);

  i = nstx_string_to_name (buf + sizeof(qh), name, 255 - sizeof(qh));
  if (i < 0)
    return -1;
  p += i;

  *p++ = 0;
  *p++ = 16;
  *p++ = 0;
  *p++ = 1;

  send (sfd, buf, (int)(p - buf), 0);
  
  return 0;
}

