/* ----------------------------------------------------------------------------
    NSTX -- tunneling network-packets over DNS

     (C) 2000 by Florian Heinz and Julien Oster

    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software
    Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.

  -------------------------------------------------------------------------- */

#include <stdio.h>
#include <string.h>
#include <unistd.h>
#include <sys/types.h>
#include <sys/socket.h>
#include <fcntl.h>
#include <netinet/in.h>
#include <errno.h>
#include <time.h>
#include <stdlib.h>

#include "nstx.h"
#include "nstx_pstack.h"

#define BUFLEN 2000

static char * domainname;

char * nstx_getpacket (int *len);
void nstx_usepacket(char *ptr, int len);
static void split_and_queue_packet(char *buf, int len);
static void copy_and_queue_data(char *buf, int len, int seq, int frc, int id);
static struct nstx_senditem * alloc_senditem(void);
extern int nstx_server_send_packet(char *, int, struct sockaddr_in *);

int nsfd = -1;
static int tapfd = -1;

int main (int argc, char *argv[]) {
   char *ptr;
   int len;
   
   if (argc < 2) {
     fprintf (stderr, "usage: %s <domainname>\n", argv[0]);
     fprintf (stderr, "example: %s tun.yomama.com\n", argv[0]);
      exit (2);
   }

   domainname = nstx_remove_dots(argv[1]);

   init_pstack(CHUNKLEN);
   
   while (1) {
      ptr = nstx_getpacket(&len);
      if (!ptr)
	continue;
      nstx_handlepacket(ptr, len, &nstx_usepacket);
   }
   
   exit(0);
}

char * nstx_getpacket (int *len) {
   static int fd, first = 1, tmpfd;
   int peerlen, msglen, hostlen = 0, newlen;
   struct sockaddr_in mysock, peer;
   unsigned char buf[BUFLEN], stripped[BUFLEN];
   static unsigned char newbuf[(BUFLEN*4)/3];
   fd_set readset;

   peerlen = sizeof(struct sockaddr_in);
   
   if (first) {
      first = 0;
      tapfd = open("/dev/tap0", O_RDWR);
      if (tapfd < 0) {
	 perror("tapdevice-open");
	 exit(1);
      }
      if ((fd = socket(AF_INET, SOCK_DGRAM, 0)) < 0) {
	 perror("socket");
	 exit(1);
      }
      mysock.sin_family = AF_INET;
      mysock.sin_port = htons(53);
      mysock.sin_addr.s_addr = INADDR_ANY;
      if (bind (fd, (struct sockaddr *) &mysock, sizeof(struct sockaddr_in))) {
	 perror("bind");
	 exit(1);
      }
      nsfd = fd;
   }
   FD_ZERO(&readset);
   FD_SET(fd,    &readset);
   FD_SET(tapfd, &readset);
   
#define MAX(a,b) ((a>b)?a:b)
   select(MAX(fd, tapfd) + 1, &readset, NULL, NULL, NULL);
#undef MAX
   
   if (FD_ISSET(fd, &readset)) {
      if ((msglen = recvfrom(fd, buf, BUFLEN, 0, (struct sockaddr *) &peer, &peerlen)) < 0) {
	 perror("recvfrom");
	 exit(1);
      }
      
      nstx_server_send_packet(buf, msglen, &peer);
      
      stripped[0] = '\0';
      
	{
	   char *component, *ptr;
	   int chunklen;
	   ptr = buf + 12;
	   component = ptr + 1;
	   
	   chunklen = *ptr;
	   while (chunklen) {
	      memcpy(stripped + hostlen, ptr+1, chunklen);
	      hostlen += chunklen;
	      ptr = ptr + chunklen + 1;
	      chunklen = *ptr;
	   }
	   hostlen -= strlen(domainname);
	   stripped[hostlen] = '\0';
	}
      
      if ((newlen = nstx_decode(newbuf, stripped, hostlen)) < 0) {
	 *len = 0;
	 return NULL;
      }
      *len = newlen;
      return newbuf;
   } else if (FD_ISSET(tapfd, &readset)) {
      newlen = read(tapfd, buf, BUFLEN);
      if (newlen < 0) {
	 perror("read");
      }
      tmpfd = open("/tmp/tap", O_CREAT|O_TRUNC|O_WRONLY);
      write(tmpfd, buf, newlen);
      close(tmpfd);
      split_and_queue_packet(buf, newlen);
   }
   *len = 0;
   return NULL;
}


void nstx_usepacket(char *ptr, int len) {
//   int fd;
//   static unsigned int id = 0;
//   char path[20];
   
//   snprintf(path, 19, "/tmp/pkt%d", id++);
//   path[19] = '\0';
//   printf("Added new packetfile %s\n", path);
   
//   fd = open(path, O_CREAT|O_WRONLY);
//   if (fd < 0) {
//      perror("open");
//      exit(1);
//   }
//   write(fd, ptr, len);
//   close(fd);
   if (tapfd < 0) {
      puts("No tap!");
     return;
   }
   write(tapfd, ptr, len);
   return;
}


struct nstx_senditem * nstx_sendlist = NULL;

static void split_and_queue_packet(char *buf, int len) {
   int i = 0, seq = 0, frc, last;
   static int id = 0;
   
   id++;
   
   frc = len / SENDLEN - 1;
   if (len % SENDLEN)
     frc++;
   
   while ((len - i) >= SENDLEN) {
      copy_and_queue_data(buf+i, SENDLEN, seq, frc, id);
      seq++;
      i += SENDLEN;
   }
   if (i != len) {
      last = (len % SENDLEN);
      copy_and_queue_data(buf+i, last, seq, frc, id);
   }
}

static void copy_and_queue_data(char *buf, int len, int seq, int frc, int id) {
   struct nstx_senditem *item;
   struct nstxhdr nstx;
   
   nstx.magic = NSTX_MAGIC;
   nstx.seq = seq;
   nstx.frc = frc;
   nstx.id = id;
   nstx.crop = 0;
   nstx.flags = 0;
   
   item = alloc_senditem();
   item->data[0] = len + sizeof(struct nstxhdr);
   memcpy(item->data + 1, &nstx, sizeof(struct nstxhdr));
   memcpy(item->data + 1 + sizeof(struct nstxhdr), buf, len);
}

static struct nstx_senditem * alloc_senditem(void) {
   struct nstx_senditem *ptr = nstx_sendlist;
   struct nstxhdr *hdr;
   struct nstxhdr tmphdr;

   if (!nstx_sendlist) {
      ptr = nstx_sendlist = malloc(sizeof(struct nstx_senditem));
   } else {
      while (ptr->next) {
	hdr = (struct nstxhdr *) ((char *)ptr->data+1);
	memcpy(&tmphdr, hdr, sizeof(struct nstxhdr));
	tmphdr.flags = NSTX_MF;
	memcpy(hdr, &tmphdr, sizeof(struct nstxhdr));
	ptr = ptr->next;
      }
      hdr = (struct nstxhdr *) ((char *)ptr->data+1);
      memcpy(&tmphdr, hdr, sizeof(struct nstxhdr));
      tmphdr.flags = NSTX_MF;
      memcpy(hdr, &tmphdr, sizeof(struct nstxhdr));
      ptr->next = malloc(sizeof(struct nstx_senditem));
      ptr = ptr->next;
   }

   memset(ptr, 0, sizeof(struct nstx_senditem));
   
   return ptr;
}

struct nstx_senditem * nstx_get_senditem(void) {
   struct nstx_senditem *ptr = nstx_sendlist;
   
   if (!nstx_sendlist)
     return NULL;
   
   ptr = nstx_sendlist;
   nstx_sendlist = nstx_sendlist->next;
   
   return ptr;
}
